@@ -15,6 +15,7 @@ import os
 import sys
 import time
 import shutil
+import pprint
 
 if __name__ == "__main__":
     os.environ["DJANGO_SETTINGS_MODULE"] = "phpBB2DjangoBB_project.settings"
@@ -37,12 +38,15 @@ from djangobb_forum.models import Category, Forum, Profile, TZ_CHOICES, Post, To
     Attachment
 from djangobb_forum import signals as djangobb_signals
 
-from django_phpBB3.models import Forum as phpbb_Forum
-from django_phpBB3.models import Topic as phpbb_Topic
-from django_phpBB3.models import Group as phpbb_Group
-from django_phpBB3.models import User as phpbb_User
-from django_phpBB3.models import Post as phpbb_Post
 from django_phpBB3.models import Attachment as phpbb_Attachment
+from django_phpBB3.models import Forum as phpbb_Forum
+from django_phpBB3.models import Group as phpbb_Group
+from django_phpBB3.models import Post as phpbb_Post
+from django_phpBB3.models import Topic as phpbb_Topic
+from django_phpBB3.models import User as phpbb_User
+from django_phpBB3.unsupported_models import get_topic_watch
+from django_phpBB3.utils import ProcessInfo, human_duration
+
 
 
 def disable_auto_fields(model_class):
@@ -102,8 +106,8 @@ class Command(BaseCommand):
         self.stdout.write("\nmigration done.\n")
 
     def _warn(self, msg):
-        self.stdout.write(self.style.ERROR(msg))
-        self.stdout.flush()
+        self.stderr.write(self.style.ERROR(msg))
+        self.stderr.flush()
 
     def check_attachment_path(self):
         path = getattr(settings, "PHPBB_ATTACHMENT_PATH", None)
@@ -291,18 +295,29 @@ class Command(BaseCommand):
     def migrate_topic(self, user_dict, forum_dict):
         self.stdout.write("\n *** Migrate phpBB topic entries...\n")
 
+        self.stdout.write("\tget topic watch information...")
+        self.stdout.flush()
+        topic_watch = get_topic_watch()
+        self.stdout.write("OK\n")
+        self.stdout.flush()
+
         topic_dict = {}
         topics = phpbb_Topic.objects.all().order_by("time")
         total = topics.count()
+        process_info = ProcessInfo(total, use_last_rates=4)
         count = 0
         start_time = time.time()
         next_status = start_time + 0.25
         for topic in topics:
             count += 1
             if time.time() > next_status:
-                self.stdout.write("\r\t%i/%i topics migrated...          " % (count, total))
-                self.stdout.flush()
                 next_status = time.time() + 1
+                rest, eta, rate = process_info.update(count)
+                msg = (
+                    "\r\t%i/%i topics migrated... rest: %i - eta: %s (rate: %.1f/sec)         "
+                ) % (count, total, rest, eta, rate)
+                self.stdout.write(msg)
+                self.stdout.flush()
 
             if topic.moved():
                 # skip moved topics -> DjangoBB doesn't support them
@@ -326,15 +341,24 @@ class Command(BaseCommand):
                 views=topic.views,
                 sticky=sticky,
                 closed=topic.locked(),
-                #subscribers=, # FIXME: ForumWatch / TopicWatch models are unsupported
 
                 # These attributes would be set later in update_topic_stats():
                 # updated, post_count, last_post
             )
+            if topic.id in topic_watch:
+                subscribers = [user_dict[user_id] for user_id in topic_watch[topic.id]]
+                obj.subscribers = subscribers
+                obj.save()
+
             topic_dict[topic.id] = obj
 
         duration = time.time() - start_time
-        self.stdout.write("\n *** %i topics migrated in %isec.\n" % (count, duration))
+        rate = float(count) / duration
+        self.stdout.write(
+            "\r *** %i topics migrated in %s (rate: %.1f/sec)\n" % (
+                count, human_duration(duration), rate
+            )
+        )
         return topic_dict
 
 
@@ -343,15 +367,20 @@ class Command(BaseCommand):
 
         posts = phpbb_Post.objects.all().order_by("time")
         total = posts.count()
+        process_info = ProcessInfo(total, use_last_rates=4)
         count = 0
         start_time = time.time()
         next_status = start_time + 0.25
         for phpbb_post in posts:
             count += 1
             if time.time() > next_status:
-                self.stdout.write("\r\t%i/%i posts migrated...          " % (count, total))
-                self.stdout.flush()
                 next_status = time.time() + 1
+                rest, eta, rate = process_info.update(count)
+                msg = (
+                    "\r\t%i/%i posts migrated... rest: %i - eta: %s (rate: %.1f/sec)         "
+                ) % (count, total, rest, eta, rate)
+                self.stdout.write(msg)
+                self.stdout.flush()
 
             topic = topic_dict[phpbb_post.topic.id]
             user = user_dict[phpbb_post.poster.id]
@@ -363,24 +392,32 @@ class Command(BaseCommand):
                 updated = None
                 updated_by = None
 
-            post = Post.objects.create(
-                topic=topic,
-                user=user,
-                created=phpbb_post.create_datetime(),
-                updated=updated,
-                updated_by=updated_by,
-                markup="bbcode",
-                body=phpbb_post.get_cleaned_bbcode(),
-                #body_html=html, # would be generated in save()
-                user_ip=phpbb_post.poster_ip,
-            )
+            try:
+                post = Post.objects.create(
+                    topic=topic,
+                    user=user,
+                    created=phpbb_post.create_datetime(),
+                    updated=updated,
+                    updated_by=updated_by,
+                    markup="bbcode",
+                    body=phpbb_post.get_cleaned_bbcode(),
+                    #body_html=html, # would be generated in save()
+                    user_ip=phpbb_post.poster_ip,
+                )
+            except Exception, err:
+                msg = (
+                    "\n +++ ERROR: creating Post entry for phpBB3 post (ID: %s):\n"
+                    "%s\n"
+                ) % (phpbb_post.id, err)
+                self._warn(msg)
+                continue
 
             if phpbb_post.has_attachment():
                 # copy attachment files
                 phpbb_attachment = phpbb_Attachment.objects.get(post_msg=phpbb_post)
                 src_path = os.path.join(settings.PHPBB_ATTACHMENT_PATH, phpbb_attachment.physical_filename)
                 if not os.path.isfile(src_path):
-                    self._warn("\n +++ ERROR: Attachment not found: '%s'" % src_path)
+                    self._warn("\n +++ ERROR: Attachment not found: '%s'\n" % src_path)
                 else:
                     attachment = Attachment(
                         size=phpbb_attachment.filesize,
@@ -404,21 +441,30 @@ class Command(BaseCommand):
                     self.stdout.flush()
 
         duration = time.time() - start_time
-        self.stdout.write("\n *** %i posts migrated in %isec.\n" % (count, duration))
-
+        rate = float(count) / duration
+        self.stdout.write(
+            "\r *** %i posts migrated in %s (rate: %.1f/sec)\n" % (
+                count, human_duration(duration), rate
+            )
+        )
 
     def update_topic_stats(self):
         self.stdout.write("\n *** set topic stats...\n")
 
         topics = Topic.objects.all()
         total = topics.count()
+        process_info = ProcessInfo(total, use_last_rates=4)
         start_time = time.time()
         next_status = time.time() + 0.25
         for count, topic in enumerate(topics):
             if time.time() > next_status:
-                self.stdout.write("\r\t%i/%i topics...           " % (count, total))
-                self.stdout.flush()
                 next_status = time.time() + 1
+                rest, eta, rate = process_info.update(count)
+                msg = (
+                    "\r\t%i/%i topics... rest: %i - eta: %s (rate: %.1f/sec)         "
+                ) % (count, total, rest, eta, rate)
+                self.stdout.write(msg)
+                self.stdout.flush()
 
             queryset = Post.objects.only("created", "updated").filter(topic=topic)
             topic.post_count = queryset.count()
@@ -436,8 +482,12 @@ class Command(BaseCommand):
             topic.save()
 
         duration = time.time() - start_time
-        self.stdout.write("\n *** %i topic stats set in %isec.\n" % (count, duration))
-
+        rate = float(count) / duration
+        self.stdout.write(
+            "\r *** %i topic stats set in %s (rate: %.1f/sec)\n" % (
+                count, human_duration(duration), rate
+            )
+        )
 
     def update_forum_stats(self):
         self.stdout.write("\n *** set forum stats...\n")
